from __future__ import division

import torch
from torch_geometric.utils import (accuracy, true_positive, true_negative,
                                   false_positive, false_negative, precision,
                                   recall, f1_score, mean_iou)


def test_metric():
    pred = torch.tensor([0, 0, 1, 1])
    target = torch.tensor([0, 1, 0, 1])

    assert accuracy(pred, target) == 0.5
    assert true_positive(pred, target, num_classes=2).tolist() == [1, 1]
    assert true_negative(pred, target, num_classes=2).tolist() == [1, 1]
    assert false_positive(pred, target, num_classes=2).tolist() == [1, 1]
    assert false_negative(pred, target, num_classes=2).tolist() == [1, 1]
    assert precision(pred, target, num_classes=2).tolist() == [0.5, 0.5]
    assert recall(pred, target, num_classes=2).tolist() == [0.5, 0.5]
    assert f1_score(pred, target, num_classes=2).tolist() == [0.5, 0.5]


def test_mean_iou():
    pred = torch.tensor([0, 0, 1, 1, 0, 1])
    target = torch.tensor([0, 1, 0, 1, 0, 0])

    out = mean_iou(pred, target, num_classes=2)
    assert out == (0.4 + 0.25) / 2

    batch = torch.tensor([0, 0, 0, 0, 1, 1])
    out = mean_iou(pred, target, num_classes=2, batch=batch)
    assert out[0] == (1 / 3 + 1 / 3) / 2
    assert out[1] == 0.25
